# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import enum
import typing
import uuid as _uuid

import winrt.system
from winrt._winrt_windows_ui_xaml_printing import (
    AddPagesEventArgs,
    GetPreviewPageEventArgs,
    PaginateEventArgs,
    PrintDocument,
)
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from winrt._winrt_windows_ui_xaml_printing import (
        PrintDocument_Static,
    )

__all__ = [
    "PreviewPageCountType",
    "AddPagesEventArgs",
    "GetPreviewPageEventArgs",
    "PaginateEventArgs",
    "PrintDocument",
    "AddPagesEventHandler",
    "GetPreviewPageEventHandler",
    "PaginateEventHandler",
]

class PreviewPageCountType(enum.IntEnum):
    FINAL = 0
    INTERMEDIATE = 1

AddPagesEventHandler = typing.Callable[[winrt.system.Object, AddPagesEventArgs], None]
GetPreviewPageEventHandler = typing.Callable[[winrt.system.Object, GetPreviewPageEventArgs], None]
PaginateEventHandler = typing.Callable[[winrt.system.Object, PaginateEventArgs], None]
